#include "config.h"
#include "regexp.h"
#include "about.h"
#include "appwin.h"
#include "msgwin.h"
#include "chlist.h"
#include "ulist.h"
#include "chattab.h"
#include "vysocket.h"
#include "vypacket.h"
#include "chatline.h"
#include "settings.h"
#include "status.h"
#include "settings.h"
#include "topicwin.h"
#include "sound.h"
#include "icons.h"
#include <iostream.h>
#include <qlayout.h>
#include <qdatetime.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qapplication.h>

const QString MainWindow::MAIN_CH("#Main");

MainWindow::MainWindow(): QMainWindow(NULL)/*{{{*/
{
	//setMinimumSize(480, 360);
	setUsesBigPixmaps(false);
	setCaption("VyQchat");
	setIcon(Icons::icon_user.pixmap(QIconSet::Small, true));
	
	QWidget *fr = new QWidget(this);
	setCentralWidget(fr);
		
	chattab = new ChatTab(fr);
	userslist = new UsersList(fr);
	chatline = new ChatLine(fr);
			 
	//
	// set up menus
	// ------------
	
	//
	// userslist context menu
	menu = new QPopupMenu(this);
	menu->insertItem("Message", this, SLOT(slotMenuMessage()));
	menu->insertItem(Icons::icon_private, "Private chat", this, SLOT(slotPrivate()));
	menu->insertItem(Icons::icon_info, "Request info", this, SLOT(slotReqInfo()));
	menu->insertItem("Beep", this, SLOT(slotBeep()));
	menu->insertSeparator();
	menu->insertItem("Mass Message", this, SLOT(slotMenuDummy()));
	menu->insertSeparator();
	menu->insertItem("Refresh list", this, SLOT(slotMenuForceRefresh()));
	connect(userslist, SIGNAL(contextMenuRequested(QListBoxItem *, const QPoint &)), this, SLOT(slotContextMenu(QListBoxItem*, const QPoint &)));
	connect(userslist, SIGNAL(doubleClicked(QListBoxItem *)), this, SLOT(slotMenuMessage(QListBoxItem *)));

	//
	// main menu
	QPopupMenu *status_menu = new QPopupMenu(this);
	QPopupMenu *chat_menu = new QPopupMenu(this);
	QPopupMenu *comm_menu = new QPopupMenu(this);
	chat_menu->insertItem("Status", status_menu);
	chat_menu->insertItem(Icons::icon_channel, "Channels", this, SLOT(slotChannelsDialog()));
	chat_menu->insertItem("ChangeTopic", this, SLOT(slotShowTopicDialog()));
	chat_menu->insertItem(Icons::icon_comm, "Commands", comm_menu);
	chat_menu->insertSeparator();
	chat_menu->insertItem("Exit", this, SLOT(slotMenuExit()));

	//
	// chat -> commands menu 
	comm_menu->insertItem("/me <text>", chatline, SLOT(slotEnter_me()));
	comm_menu->insertItem("/nick <new nick>", chatline, SLOT(slotEnter_nick()));
	comm_menu->insertItem("/join <channel>", chatline, SLOT(slotEnter_join()));
	comm_menu->insertItem("/leave", chatline, SLOT(slotEnter_leave()));
	comm_menu->insertItem("/chat <nick>", chatline, SLOT(slotEnter_chat()));
	comm_menu->insertItem("/topic <text>", chatline, SLOT(slotEnter_topic()));
	comm_menu->insertItem("/info <nick>", chatline, SLOT(slotEnter_info()));
	comm_menu->insertItem("/beep <nick>", chatline, SLOT(slotEnter_beep()));
	comm_menu->insertItem("/mass <nick>", chatline, SLOT(slotEnter_mass()));
	comm_menu->insertItem("/here", chatline, SLOT(slotEnter_here()));
	comm_menu->insertItem("/normal", chatline, SLOT(slotEnter_normal()));
	comm_menu->insertItem("/dnd <text>", chatline, SLOT(slotEnter_dnd()));
	comm_menu->insertItem("/away <text>", chatline, SLOT(slotEnter_away()));
	comm_menu->insertItem("/offline <text>", chatline, SLOT(slotEnter_offline()));
	comm_menu->insertItem("/help", chatline, SLOT(slotEnter_help()));
	
	//
	// chat -> status menu
	status_menu->insertItem(Icons::icon_user, "Available", this, SLOT(slotGoNORMAL()));
	status_menu->insertItem(Icons::icon_user_dnd, "DND", this, SLOT(slotDNDdialog()));
	status_menu->insertItem(Icons::icon_user_away, "Away", this, SLOT(slotAWAYdialog()));
	status_menu->insertItem(Icons::icon_user_offline, "Offline", this, SLOT(slotMenuDummy()));
	
	//
	// edit menu
	QPopupMenu *edit_menu = new QPopupMenu(this);
	edit_menu->insertItem("Clear Chat", chattab, SLOT(slotClearCurrent()));
	edit_menu->insertItem("Clear Line", chatline, SLOT(clear()));

	//
	// Options menu
        QPopupMenu *options_menu = new QPopupMenu(this);
	options_menu->insertItem(Icons::icon_settings, "Settings", this, SLOT(slotSettingsDialog()));
	options_menu->insertItem("Regexp matching", this, SLOT(slotRegexpDialog()));
	
	//
	// help menu
	QPopupMenu *help_menu = new QPopupMenu(this);
	help_menu->insertItem("About", this, SLOT(slotPopupAbout()));
	
	main_menu = menuBar();
	main_menu->insertItem("&Chat", chat_menu);
	main_menu->insertItem("&Edit", edit_menu);
	main_menu->insertItem("&Options", options_menu);
	main_menu->insertSeparator();
	main_menu->insertItem("&Help", help_menu);
	
	//
	// toolbar
	QToolBar *main_toolbar = new QToolBar(this, "Main toolbar");
	status_combo = new QComboBox(main_toolbar);
	status_combo->insertItem(Icons::icon_user.pixmap(QIconSet::Small, true), "Available");
	status_combo->insertItem(Icons::icon_user_dnd.pixmap(QIconSet::Small, true), "DND");
	status_combo->insertItem(Icons::icon_user_away.pixmap(QIconSet::Small, true), "Away");
	status_combo->insertItem(Icons::icon_user_offline.pixmap(QIconSet::Small, true), "Offline");
	connect(status_combo, SIGNAL(activated(int)), this, SLOT(slotStatusComboActivated(int)));
	QToolButton *comm_button  = new QToolButton(Icons::icon_comm, "Open File", QString::null, this, SLOT(slotMenuDummy()), main_toolbar);
	comm_button->setPopup(comm_menu);
	comm_button->setPopupDelay(0);
	QToolButton *channels_button  = new QToolButton(Icons::icon_channel, "Channels", QString::null, this, SLOT(slotChannelsDialog()), main_toolbar);
	QToolButton *conf_button  = new QToolButton(Icons::icon_settings, "Settings", QString::null, this, SLOT(slotSettingsDialog()), main_toolbar);
	
	userslist->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding));

	//
	// glowny pionowy layout
	QBoxLayout *l2 = new QVBoxLayout(fr);

	//
	// layout poziomy: okno glowne i lista userow
	QBoxLayout *l1 = new QHBoxLayout(l2);
	l1->addWidget(chattab);
	l1->addWidget(userslist);

	//
	// enter button
	QPushButton *b_enter = new QPushButton(Icons::icon_enter, "", fr);
	b_enter->setFixedWidth(Icons::icon_enter.pixmap(QIconSet::Large, true).width());
	connect(b_enter, SIGNAL(clicked()), this, SLOT(slotSendline()));

	QBoxLayout *l3 = new QHBoxLayout(l2);
	l3->addWidget(chatline, 1);
	l3->addWidget(b_enter, 0);
	
	connect(chatline, SIGNAL(returnPressed()), this, SLOT(slotSendline()));
	chatline->setFocus();

	availchannels = new ChannelList();
	dnd_list = new History("dnd", 10);
	away_list = new History("away", 10);
	offline_list = new History("offline", 10);
	regexp_list = new History("regexp", 10);
	dnd_list->load();
	away_list->load();
	offline_list->load();
	regexp_list->load();
	regexp = QRegExp(regexp_list->first());

	//
	// Load settings
	settings = new Settings();
	settings->load();
	
	nick = settings->nick;
	regexp_on = settings->regexp_on;
	status = STATUS_NORMAL;
	gender = GENDER_MALE;
	remoteactive = RA_ACTIVE; 
	special_color = QColor(0xff, 0x00, 0x00);
	normal_color = QColor(0x00, 0x00, 0xff);
	own_color = QColor(0x00, 0x00, 0x00);

	resize(settings->width, settings->height);
	if ((settings->x >= 0) && (settings->y >= 0))
		move(settings->x, settings->y);
	
	QString baddr(settings->broadcast);
	sockin = new VyGetSocket();
	if (sockin->isValid())
	{
		socknotify = new QSocketNotifier(sockin->socket(), QSocketNotifier::Read, this);
		connect(socknotify, SIGNAL(activated(int)), this, SLOT(slotDataReceived()));
	}
	else
		cerr << "sockget error" << endl;
	
	sockout = new VySendSocket(baddr);
	if (sockout->isValid())
	{
		//
		// chat session begins
		chattab->addChannel(MAIN_CH, settings->topic);
		chattab->write(MAIN_CH, "// Welcome to vyqchat v0.1.1\n"
					"// (w)by Pawel Stolowski\n"
					"// send comments to yogin@linux.bydg.org\n"
					"//\n", QColor(0xf4, 0x09, 0x15));
		sockout->vy_join(nick, MAIN_CH, status, gender);
		//sockout->vy_imhere(nick, nick, status, winstate);
		sockout->vy_whohere(nick);
	}	
	else
		cerr << "sockout error" << endl;

	//
	// set-up Timer for users-list refresh
	if (settings->refresh)
	{
		QTimer *timer = new QTimer(this);
		connect(timer, SIGNAL(timeout()), this, SLOT(slotUsersListUpdate()));
		timer->start(settings->refresh * 1000, false);
	}
	if (!Sound::init(settings))
		QMessageBox::warning(this, "VyQChat", Sound::error(), "OK");
}/*}}}*/

MainWindow::~MainWindow()/*{{{*/
{
	Sound::close();

	sockout->vy_leave(nick, MAIN_CH, gender);
	settings->x = x();
	settings->y = y();
	settings->width = width();
	settings->height = height();

	delete settings;
	delete availchannels;
	delete dnd_list;
	delete away_list;
	delete offline_list;
	delete regexp_list;
	delete chattab;
	delete chatline;
	delete userslist;
}/*}}}*/

//
// returns current time in form [hh:mm]
QString MainWindow::timestring()/*{{{*/
{
	QTime current = QTime::currentTime();
	return QString("[") + current.toString("hh:mm") + QString("] ");
}/*}}}*/

//
// parses keyboard-commands
bool MainWindow::parse(QString &str)/*{{{*/
{
	enum {
		CMD_ME,
		CMD_NORMAL,
		CMD_AWAY,
		CMD_OFFLINE,
		CMD_BEEP,
		CMD_DND,
		CMD_HERE,
		CMD_INFO,
		CMD_JOIN,
		CMD_LEAVE,
		CMD_MESSAGE,
		CMD_NICK,
		CMD_TOPIC,
		CMD_CHAT,
		CMD_HELP
	};
	const struct { QString command; char id; int argnum; } cmdlut[] = {
		{"normal", CMD_NORMAL,	0},
		{"away",   CMD_AWAY,	1},
		{"offline",CMD_OFFLINE,	1},
		{"beep",   CMD_BEEP,	1},
		{"dnd",    CMD_DND,	1},
		{"here",   CMD_HERE,	0},
		{"info",   CMD_INFO,	1},
		{"join",   CMD_JOIN,	1},
		{"j",      CMD_JOIN,	1},
		{"leave",  CMD_LEAVE,	1},
		{"mesg",   CMD_MESSAGE,	2},
		{"m",      CMD_MESSAGE,	2},
		{"nick",   CMD_NICK,	1},
		{"topic",  CMD_TOPIC,	1},
		{"chat",   CMD_CHAT,	1},
		{"me",     CMD_ME,	1},
		{"help",   CMD_HELP,	0},
		{"?",      CMD_HELP,	0},
		{NULL,	   0,		0}
	};
	char tmp[1500];
	
	str = str.right(str.length() - 1); //cut the leading slash
	QString cmd = str.section(' ', 0, 0); //take the command name

	for (int i=0; cmdlut[i].command; i++)
		if (cmdlut[i].command == cmd)
		{
			QString channel = chattab->current();
			QString arg[2];
			if (cmdlut[i].argnum==1)
				arg[0] = str.section(' ', 1);
			else
				if (cmdlut[i].argnum==2)
				{
					arg[0] = str.section(' ', 1, 1);
					arg[1] = str.section(' ', 2);
				}
			switch (cmdlut[i].id)
			{
				case CMD_ME:
					if (chattab->isPrivate())
						sockout->vy_private_me(userslist->getIP(channel), nick, channel, arg[0]);
					else
						sockout->vy_me(channel, nick, arg[0]);
					break;
				case CMD_HERE:
					sockout->vy_herereq(nick, channel);
					break;
				case CMD_INFO:
					if (userslist->exists(arg[0]))
						sockout->vy_inforeq(userslist->getIP(arg[0]), arg[0], nick);
					break;
				case CMD_BEEP:
					if (userslist->exists(arg[0]))
						 sockout->vy_beep(userslist->getIP(arg[0]), arg[0], nick);
					break;
				case CMD_MESSAGE:
					if (userslist->exists(arg[0]))
						 sockout->vy_message(userslist->getIP(arg[0]), nick, arg[0], arg[1]);
					break;
				case CMD_NICK:
					if ((!arg[0].isNull()) && (!userslist->exists(arg[0])))
					{
						sockout->vy_nickchange(nick, arg[0], gender);
						nick = arg[0];
					}
					break;
				case CMD_TOPIC:
					if (!arg[0].isNull())
						sockout->vy_newtopic(channel, arg[0], nick);
					else
					{
						sprintf(tmp, "%s Current topic on %s:\n%s", timestring().latin1(), channel.latin1(), chattab->getTopic(channel).latin1());
						chattab->write(channel, tmp, special_color);
					}
					break;
				case CMD_JOIN:	
					/*arg[0] = (arg[0][0]=='#') ?  arg[0] : "#" + arg[0];
					chattab->addChannel(arg[0], settings->topic);
					sockout->vy_join(nick, arg[0], status, gender);*/
					slotJoin(arg[0]);
					break;
				case CMD_LEAVE:
					if (channel == QString(MAIN_CH))
						qApp->quit(); //vy_leave for #main will be send in MainWindow destructor
					else
					{
						if (chattab->isPrivate())
						{
							const QString to = chattab->current();
							chattab->delPrivate(channel);
							sockout->vy_private_leave(userslist->getIP(to), nick, to, gender);
						}
						else
						{
							chattab->delChannel(channel);
							sockout->vy_leave(nick, channel, gender);
						}
					}
					break;
				case CMD_CHAT:
					if (userslist->exists(arg[0]))
					{
						chattab->addPrivate(arg[0]);
						sockout->vy_private_join(userslist->getIP(arg[0]), nick, arg[0], gender);
					}
					break;
				case CMD_NORMAL:
					slotGoNORMAL();
					break;
				case CMD_DND:
					slotGoDND(arg[0]);
					break;
				case CMD_AWAY:
					slotGoAWAY(arg[0]);
					break;
				case CMD_OFFLINE:
					slotGoOFFLINE(arg[0]);
				case CMD_HELP:
					str = "Available commands:\n";
					for (int i=0; cmdlut[i].command; i++)
					{
						str.append(cmdlut[i].command);
						str.append(' ');
					}
					chattab->write(channel, str, special_color);
					break;
				default: break;
			}
			return true;
		}
	return false;
}/*}}}*/

void MainWindow::decode(const QHostAddress &fromaddr, char *p)/*{{{*/
{
	MessageRcvWindow *msgwin;

	const char *status_lut[] = {"Normal", "DND", "Away", "Offline"};
	char tmp[1500];
	User *user;

	VyPacket vp(p, 1500);
	
	switch (vp.ptype)
	{
		/* WHO HERE
		 * updater */
		case VyPacket::VyWhoHere:
			sockout->vy_imhere(vp.from, nick, status, winstate);
			break;
		/* IM HERE
		 * updater :: responder :: status + remote active */
		case VyPacket::VyImHere:
			if (!userslist->exists(vp.from))
			{
				  //user->setRemoteActive(ptrs[2][1]);
				  userslist->addUser(vp.from, 0xff, vp.status, fromaddr);
			}
			else
				userslist->incDeadCounter(vp.from);
			break;
		/* SIMPLE CHAT
		 * channel :: author :: text */
		case VyPacket::VySay:  
			      if (chattab->exists(vp.channel))
			      {
				      sprintf(tmp, "<%s> %s", vp.from, vp.text);
				      chattab->write(vp.channel, tmp, (QString(vp.from) == nick) ? own_color : normal_color);
				      
				      //
				      // regexps are mateched for any channel you are on
				      // play regexpmatch-sound
				      // write alert message on current channel
				      if (regexp_on && QString(vp.text).contains(regexp))
				      {
					      Sound::play(Sound::RegexMatch);
					      chattab->write(chattab->current(), "Regular expression matched on " + QString(vp.channel), special_color);
				      }
				      else
				      {
					      //
					      // play chatline-sound if current channel
					      if (strcmp(chattab->current().latin1(), vp.channel)==0)
						      Sound::play(Sound::Chatline);
				      }
			      }
			      break;
		/* ME
		 * channel :: author :: text */
		case VyPacket::VyMe:
			    sprintf(tmp, "*%s* %s", vp.from, vp.text);
			    chattab->write(vp.channel, tmp, special_color);
			    break;
		/* NICK CHANGE
		 * old :: new :: gender */
		case VyPacket::VyNickChange: 
				    sprintf(tmp, "%s<%s> is known as <%s> from now", timestring().latin1(), vp.from, vp.to);
				    chattab->write(MAIN_CH, tmp, special_color);
				    userslist->newNick(vp.from, vp.to);
			  break;
		  /* BEEP SIGNAL
		   * to :: from  */
		  case VyPacket::VyBeepSignal:
				       sprintf(tmp, "Beep from %s received", vp.from);
				       chattab->write(MAIN_CH, tmp, special_color);
				       sockout->vy_beepack(fromaddr, vp.from, nick, gender);
				       Sound::play(Sound::Beep);
				       break;
		  /* BEEP ACK
		   * to :: from :: gender */
		  case VyPacket::VyBeepAck: 
				    sprintf(tmp, "%s received your beep",  vp.from);
				    chattab->write(MAIN_CH, tmp, special_color);
				    break;

		/* JOIN
		 * who :: channel :: status + gender */
		  case VyPacket::VyJoin:
			      if ((user = userslist->getUser(vp.from))==NULL)
				      user = userslist->addUser(vp.from, vp.gender, vp.status, fromaddr);
			      if (chattab->exists(vp.channel))
			      {
				      //
				      // send current topic to new user (if it's not me)
			      	      if (strcmp(vp.from, nick.latin1())!=0)
					      sockout->vy_currenttopic(fromaddr, vp.from, vp.channel, chattab->getTopic(vp.channel));
				      sprintf(tmp, "%s%s joins channel %s (%s mode)", timestring().latin1(), vp.from, vp.channel, status_lut[user->getStatus()-STATUS_NORMAL]);
				      chattab->write(vp.channel, tmp, special_color);
			      }
			      Sound::play(Sound::JoinChannel);
			      availchannels->add(vp.channel);
			      break;
		/* LEAVE
		 * who :: channel :: gender */
		case VyPacket::VyLeave:
			  sprintf(tmp, "%s%s leaves channel %s", timestring().latin1(), vp.from, vp.channel);
			  chattab->write(vp.channel, tmp, special_color);
			  if (strcmp(vp.channel, MAIN_CH.latin1())==0)
				  userslist->delUser(vp.from);
			  Sound::play(Sound::LeaveChannel);
			  break;

		  /* PRIVATE JOIN
		   * from :: to :: gender */
		  case VyPacket::VyPrivateJoin: 
			  { QString prvname = QString(vp.from)==nick ? vp.to : vp.from;
			  		sprintf(tmp, "%s%s joins private", timestring().latin1(), vp.from);
					chattab->write(prvname, tmp, special_color); }
					break;
		  /* PRIVATE LEAVE
		   * from :: to :: gender */
		  case VyPacket::VyPrivateLeave: {
					 QString prvname = QString(vp.from)==nick ? vp.to : vp.from;
					 sprintf(tmp, "%s%s leaves private", timestring().latin1(), vp.from);
					 chattab->write(prvname, tmp, special_color); }
					 break;
		  /* from :: to :: text */
		  case VyPacket::VyPrivateSay: {
				       QString prvname = QString(vp.from)==nick ? vp.to : vp.from;
				       sprintf(tmp, "<%s> %s", vp.from, vp.text);
				       chattab->write(prvname, tmp, (QString(vp.from) == nick) ? own_color : normal_color); }
				       break;
		  /* from :: to :: text */
		  case VyPacket::VyPrivateMe:  {
				       QString prvname = QString(vp.from)==nick ? vp.to : vp.from;
				       sprintf(tmp, "*%s* %s", vp.from, vp.text);
				       chattab->write(prvname, tmp, special_color); 
				       if (regexp_on && QString(vp.text).contains(regexp))
				       {
					       Sound::play(Sound::RegexMatch);
					       chattab->write(chattab->current(), "Regular expression matched on " + QString(prvname), special_color);
				       }}
				       break;
		/* MESSAGE
		 * from :: to :: text */
		case VyPacket::VyMessage: if (status==STATUS_OFFLINE) break;
				 sockout->vy_messageack(fromaddr, status, vp.from, nick, gender, autoanswer);
				 msgwin = new MessageRcvWindow(vp.from, vp.text);
				 connect(msgwin, SIGNAL(reply(const QString &, const QString &)), this, SLOT(slotSendMessage(const QString &, const QString &)));
				 msgwin->show();
				 break;
		/* MESSAGEACK
		 * status + whorec :: towhoack :: gender + currentaa */	  
		case VyPacket::VyMessageAck: sprintf(tmp, "%s%s received your message", timestring().latin1(), vp.from);
				    chattab->write(MAIN_CH, tmp, special_color);
			  break;

		/* HERE REQUEST
		 * from :: channel */
		case VyPacket::VyHereReq:
				 sockout->vy_hereack(fromaddr, vp.from, vp.channel, nick, remoteactive);
			  break;
		/* HERE ACK 
		 * to :: channel :: from :: remote active */
		case VyPacket::VyHereReqAck:
				 sprintf(tmp, "%sHere %s (%s)", timestring().latin1(), vp.from, remoteactive==WIN_ACTIVE ? "active":"not active");
				 chattab->write(vp.channel, tmp, special_color);
			  break;
		/* INFO REQUEST
		 * to :: from */
		case VyPacket::VyInfoReq:
			  sprintf(tmp, "%s Info requested from %s", timestring().latin1(), vp.from);
			  chattab->write(MAIN_CH, tmp, special_color);
			  sockout->vy_inforeqack(userslist->getIP(vp.from), vp.from, nick, settings->hostname, nick, settings->ip, settings->hide_channels==true ? QString("") : chattab->channelsList(), (status!=STATUS_NORMAL) ? autoanswer : QString(" "));
			  break;
		/* INFO REQUEST ACK
		 * to :: from :: computer :: name :: ip :: channels :: answer */
		case VyPacket::VyInfoReqAck: { 
				    sprintf(tmp, "%s Info on %s:\nName: %s Host: %s [%s]\nChannels: %s",
						    timestring().latin1(),
						    vp.from, vp.name, vp.computer, fromaddr.toString().latin1(),
						    vp.channel);
				    chattab->write(chattab->current(), tmp, special_color);
				    if (*vp.text)
				    {
					    sprintf(tmp, "%s", vp.text);
					    chattab->write(chattab->current(), tmp, special_color);
					    User *user = userslist->getUser(vp.from); //bylo name
					    /*if (user)
					    {
						    printf("%s\n", ptrs[6]);
						    user->setToolTipInfo(ptrs[6]);
					    }*/
				    }
				    QStringList l; // ????
				    availchannels->add(l);
				    }
				    break;
		/* CURRENT TOPIC
		 * towho :: channel :: topic */
		case VyPacket::VyCurrTopic:
				   if (chattab->getTopic(vp.channel).isEmpty() && *vp.topic!=0)
				   {
					   sprintf(tmp, "%s Current topic on %s:\n%s", timestring().latin1(),
							   vp.channel, vp.topic);
					   chattab->setTopic(vp.channel, vp.topic);
					   chattab->write(vp.channel, tmp, special_color);
				   }
				   break;
	    	/* NEW TOPIC
		 * channel :: topic */
		case VyPacket::VyNewTopic:
				  if (chattab->setTopic(vp.channel, vp.topic))
				  {
					  sprintf(tmp, "%s New topic:\n%s", timestring().latin1(), vp.topic);
					  chattab->write(vp.channel, tmp, special_color);
				  }
				 break;
		/* CHANGE STATUS
		 * who :: status + gender + aa */
		case VyPacket::VyChangeStat:
				    if (userslist->setStatus(vp.from, vp.status))
				    {
					    sprintf(tmp, "%s%s enters %s mode", timestring().latin1(),
							    vp.from, status_lut[vp.status - STATUS_NORMAL]);
					    chattab->write(MAIN_CH, tmp, special_color);
					    if ((vp.status!=STATUS_NORMAL) && *vp.answer)
					    {
						    sprintf(tmp, "%s", vp.answer);
						    chattab->write(MAIN_CH, tmp, special_color);
					    }
				    }
				    break;
		default:   /* unknown packet */
			  break;
	}
}/*}}}*/

/*-----------------------------------------------------------------------------------------------------
 * 
 *                                            S L O T S
 *
 -----------------------------------------------------------------------------------------------------*/

void MainWindow::slotSendline()/*{{{*/
{
	QString str = chatline->text();
	if (!str.isEmpty())
	{
		if (str[0]=='/')
		{
			if (!parse(str))
				chattab->write(MAIN_CH, "unknown command", special_color);
		}
		else
		{
			if (chattab->isPrivate())
			{
				char tmp[1500];
				const QString to = chattab->current();
				sockout->vy_private_say(userslist->getIP(to), nick, to, str);
				//
				// if reciever is not me, write message directly to widget
				// (because we won't get it back from socket)
				if (to!=nick)
				{
					sprintf(tmp, "<%s> %s", nick.latin1(), str.latin1());
					chattab->write(to, tmp, own_color);
				}
			}
			else
				sockout->vy_say(chattab->current(), nick, str);
		}
		chatline->clear();
	}
}/*}}}*/

void MainWindow::slotDataReceived()/*{{{*/
{
	char buffer[VY_MAXPACKETSIZE+1];
	sockin->readBlock(buffer, VY_MAXPACKETSIZE);
	QHostAddress addr = sockin->peerAddress();
	decode(addr, buffer);
}/*}}}*/

void MainWindow::slotSendMessage(const QString &to, const QString &msg)/*{{{*/
{
	if (userslist->exists(to))
		sockout->vy_message(userslist->getIP(to), nick, to, msg);
}/*}}}*/

void MainWindow::slotMenuExit()/*{{{*/
{
	close();
}/*}}}*/

void MainWindow::slotMenuForceRefresh()/*{{{*/
{
	userslist->clear();
	sockout->vy_whohere(nick);
}/*}}}*/

void MainWindow::slotMenuDummy()/*{{{*/
{
	cout << "dummy" << endl;
}/*}}}*/

void MainWindow::slotContextMenu(QListBoxItem *it, const QPoint &p)/*{{{*/
{
	if (it)
		menu->popup(p);
}/*}}}*/

void MainWindow::slotMenuMessage()/*{{{*/
{
	int i = userslist->currentItem();
	if (i>=0)
	{
		QString who = userslist->text(i);
		MessageSndWindow *msgwin = new MessageSndWindow(who);
		connect(msgwin, SIGNAL(send(const QString &, const QString &)), this, SLOT(slotSendMessage(const QString &, const QString &)));
		msgwin->show();
	}
}/*}}}*/

void MainWindow::slotMenuMessage(QListBoxItem *it)/*{{{*/
{
	if (it)
	{
		QString who = it->text();
		MessageSndWindow *msgwin = new MessageSndWindow(who);
		connect(msgwin, SIGNAL(send(const QString &, const QString &)), this, SLOT(slotSendMessage(const QString &, const QString &)));
		msgwin->show();
	}
}/*}}}*/

void MainWindow::slotReqInfo()/*{{{*/
{
	int i = userslist->currentItem();
	if (i>=0)
	{
		QString who = userslist->text(i);
		sockout->vy_inforeq(userslist->getIP(who), who, nick);
	}
}/*}}}*/

void MainWindow::slotPrivate()/*{{{*/
{
	int i = userslist->currentItem();
	if (i>=0)
	{
		QString who = userslist->text(i);
		chattab->addPrivate(who);
		sockout->vy_private_join(userslist->getIP(who), nick, who, gender);
	}
}/*}}}*/

void MainWindow::slotBeep()/*{{{*/
{
	int i = userslist->currentItem();
	if (i>=0)
	{
		QString who = userslist->text(i);
		sockout->vy_beep(userslist->getIP(who), who, nick);
	}
}/*}}}*/

void MainWindow::slotJoin(const QString &channel)/*{{{*/
{
	if (channel.isEmpty())
		return;
	
	QString name = (channel[0]=='#') ? channel : '#' + channel;

	if (!chattab->exists(name))
	{
		chattab->addChannel(name, settings->topic);
		sockout->vy_join(nick, name, status, gender);
	}
}/*}}}*/

void MainWindow::slotShowTopicDialog()/*{{{*/
{
	QString channel = chattab->current();
	if (!chattab->isPrivate(channel))
	{
		QString topic = chattab->getTopic(channel);
		topic = topic.left(topic.findRev('(')-1);
		TopicWin *w = new TopicWin(this, channel, topic);
		connect(w, SIGNAL(newTopic(const QString &, const QString &)), this, SLOT(slotNewTopic(const QString &, const QString &)));
		w->show();
	}
}/*}}}*/

void MainWindow::slotSettingsDialog()/*{{{*/
{
	SettingsDialog *s = new SettingsDialog(this, settings);
	s->show();
}/*}}}*/

void MainWindow::slotGoNORMAL()/*{{{*/
{
	status_combo->setCurrentItem(0);
	sockout->vy_changestatus(nick, status=STATUS_NORMAL, gender, "" /*QString::null*/);
}/*}}}*/

void MainWindow::slotGoDND(const QString &txt)/*{{{*/
{
	dnd_list->append(txt);
	sockout->vy_changestatus(nick, status=STATUS_DND, gender, autoanswer=txt);
}/*}}}*/

void MainWindow::slotGoAWAY(const QString &txt)/*{{{*/
{
	away_list->append(txt);
	sockout->vy_changestatus(nick, status=STATUS_AWAY, gender, autoanswer=txt);
}/*}}}*/

void MainWindow::slotGoOFFLINE(const QString &txt)/*{{{*/
{
	offline_list->append(txt);
	sockout->vy_changestatus(nick, status=STATUS_OFFLINE, gender, autoanswer=txt);
}/*}}}*/

void MainWindow::slotRegexpDialog()/*{{{*/
{
	RegexpWin *s = new RegexpWin(this, *regexp_list, regexp_on);
	connect(s, SIGNAL(changeRegexp(const QString &, bool)), this, SLOT(slotChangeRegexp(const QString &, bool)));
	s->show();
}/*}}}*/

void MainWindow::slotDNDdialog()/*{{{*/
{
	StatusWin *s = new StatusWin(this, "Enter DND mode....", *dnd_list);
	connect(s, SIGNAL(changeStatus(const QString &)), this, SLOT(slotGoDND(const QString &)));
	status_combo->setCurrentItem(1);
	s->show();
}/*}}}*/

void MainWindow::slotAWAYdialog()/*{{{*/
{
	StatusWin *s = new StatusWin(this, "Enter Away mode....", *away_list);
	connect(s, SIGNAL(changeStatus(const QString &)), this, SLOT(slotGoAWAY(const QString &)));
	status_combo->setCurrentItem(2);
	s->show();
}/*}}}*/

void MainWindow::slotOFFLINEdialog()/*{{{*/
{
	StatusWin *s = new StatusWin(this, "Enter Offline mode....", *offline_list);
	connect(s, SIGNAL(changeStatus(const QString &)), this, SLOT(slotGoOFFLINE(const QString &)));
	status_combo->setCurrentItem(3);
	s->show();
}/*}}}*/

void MainWindow::slotNewTopic(const QString &channel, const QString &txt)/*{{{*/
{
	sockout->vy_newtopic(channel, txt, nick);
}/*}}}*/

void MainWindow::slotChangeRegexp(const QString &txt, bool on)/*{{{*/
{
	regexp_list->append(txt);
	regexp = QRegExp(txt);
	regexp_on = (regexp.isValid() && on); 
	settings->regexp_on = regexp_on;
}/*}}}*/

void MainWindow::slotStatusComboActivated(int item)/*{{{*/
{
	switch (item)
	{
		case 0: slotGoNORMAL(); break;
		case 1: slotDNDdialog(); break;
		case 2: slotAWAYdialog(); break;
		case 3: slotOFFLINEdialog(); break;
		default: break;
	}
}/*}}}*/

void MainWindow::slotUsersListUpdate()/*{{{*/
{
	userslist->decDeadCounters();
	sockout->vy_whohere(nick);
}/*}}}*/

void MainWindow::slotPopupAbout()/*{{{*/
{
	AboutDialog *about = new AboutDialog(this);
	about->show();
}/*}}}*/

void MainWindow::slotChannelsDialog()/*{{{*/
{
	ChannelsDialog *c = new ChannelsDialog(this, availchannels);
	connect(c, SIGNAL(joinChannel(const QString &)), this, SLOT(slotJoin(const QString &)));
	c->show();
}/*}}}*/

