#ifndef __APPWIN_H
#define __APPWIN_H

#include <qmainwindow.h>
#include <qsocketnotifier.h>
#include <qhostaddress.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qregexp.h>
#include "history.h"
#include <qptrvector.h>

class ChannelList;
class Settings;
class User;
class UsersList;
class ChatTab;
class ChatLine;
class VySendSocket;
class VyGetSocket;

typedef QPtrVector<char> CharPtrVector;

class MainWindow: public QMainWindow
{
	Q_OBJECT
	private slots:
		void slotSendMessage(const QString &to, const QString &msg);
		void slotReqInfo();
		void slotPrivate();
		void slotJoin(const QString &);
		void slotBeep();
		void slotSendline();
		void slotDataReceived();
		void slotMenuExit();
		void slotMenuForceRefresh();
		void slotMenuDummy();
		void slotContextMenu(QListBoxItem*, const QPoint &);
		void slotShowTopicDialog();
		void slotMenuMessage();
		void slotMenuMessage(QListBoxItem *it);
		void slotSettingsDialog();
		void slotGoNORMAL();
		void slotGoDND(const QString &txt);
		void slotGoAWAY(const QString &txt);
		void slotGoOFFLINE(const QString &txt);
		void slotRegexpDialog();
		void slotDNDdialog();
		void slotAWAYdialog();
		void slotOFFLINEdialog();
		void slotNewTopic(const QString &channel, const QString &txt);
		void slotChangeRegexp(const QString &txt, bool on);
		void slotStatusComboActivated(int item);
		void slotUsersListUpdate();
		void slotPopupAbout();
		void slotChannelsDialog();
	private:
		QMenuBar *main_menu;
		QComboBox *status_combo;
		QPopupMenu *menu; //userslist context-menu
		ChatLine *chatline;
		ChatTab *chattab;
		UsersList *userslist;
		VySendSocket *sockout;
		VyGetSocket *sockin;
		QSocketNotifier *socknotify;
		static const QString MAIN_CH;
		//
		// current options and status
		ChannelList *availchannels;
		History *dnd_list;
		History *away_list;
		History *offline_list;
		History *regexp_list;
		Settings *settings;
		QString nick;
		QString autoanswer;
		QColor special_color;
		QColor normal_color;
		QColor own_color;
		QRegExp regexp;
		bool regexp_on;
		char status;
		char winstate;
		char gender;
		char remoteactive;

		QString timestring();
		CharPtrVector split(char *buff, int num);
		bool parse(QString &str);
		void decode(const QHostAddress &fromaddr, char *p);
	public:
		
		MainWindow();
		~MainWindow();
};

#endif

