#include "config.h"
#include "settings.h"
#include <stdlib.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qvalidator.h>
#include <qhostaddress.h>
#include <qradiobutton.h>
#include <qvbox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>

#include <unistd.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <arpa/inet.h>

#ifndef DATADIR
#define DATADIR "./"
#endif

extern bool amIroot();

Settings::Settings()/*{{{*/
{
}/*}}}*/

Settings::~Settings()/*{{{*/
{
	save();
}/*}}}*/

bool Settings::getDefaults()/*{{{*/
{
	QString datapath(DATADIR);

	bool status = false;
	def_topic = "no topic";
	def_sound = Sound::PCSpeaker;
	def_snd_chatline = datapath + "chatline.wav";
	def_snd_beep = datapath + "beep.wav";
	def_snd_join = datapath + "join.wav";
	def_snd_leave = datapath + "leave.wav";
	def_snd_message = datapath + "beep.wav";
	def_snd_regexp = datapath + "beep.wav";

	//
	// get hostname
	char tmp[255];
	gethostname(tmp, 254);
	def_hostname = QString(tmp);

	//
	// get username
	char *login = getenv("LOGNAME");
	def_nick = QString(login);
	
	//
	// get eth0 parameters
	def_broadcast = "127.255.255.255";
	def_ip = "127.0.0.1";

	struct ifconf ifc;
	int sockfd = socket(AF_INET, SOCK_DGRAM, 0);

	ifc.ifc_req = new ifreq [16];
	ifc.ifc_len = 16*sizeof(struct ifreq);

	if (ioctl(sockfd, SIOCGIFCONF, &ifc)==0)
	{
		status = true;

		const int num = ifc.ifc_len/sizeof(struct ifreq);
		struct ifreq *req = ifc.ifc_req; 
		for (int i=0; i<num; i++)
		{
			if (strcmp(req[i].ifr_name, "eth0")==0)
			{
				if (ioctl(sockfd, SIOCGIFBRDADDR , &req[i])==0) 
					def_broadcast = QString(inet_ntoa(*(in_addr *)(&req[i].ifr_broadaddr.sa_data[2])));
				else
					status = false;
				
				if (ioctl(sockfd, SIOCGIFADDR , &req[i])==0) 
					def_ip = QString(inet_ntoa(*(in_addr *)(&req[i].ifr_addr.sa_data[2])));
				else
					status = false;
			}
		}
		
	}
	delete [] ifc.ifc_req;
	close(sockfd);
	return status;
}/*}}}*/

void Settings::load()/*{{{*/
{
	getDefaults();

	QSettings cfg;
	nick = cfg.readEntry("/vyqchat/General/Nick", def_nick);
	hostname = cfg.readEntry("/vyqchat/General/Hostname", def_hostname);
	ip = cfg.readEntry("/vyqchat/General/IP", def_ip);
	broadcast = cfg.readEntry("/vyqchat/General/Broadcast", def_broadcast);
	refresh = cfg.readNumEntry("/vyqchat/General/Refresh", 300);
	hide_channels = cfg.readBoolEntry("/vyqchat/General/HideChannels", false);
	regexp_on = cfg.readBoolEntry("/vyqchat/General/RegExpOn", false);
	topic = cfg.readEntry("/vyqchat/General/Topic", def_topic);
	sound = (Sound::SoundType)cfg.readNumEntry("/vyqchat/Sound/sound", def_sound);
	
	snd_chatline = cfg.readEntry("/vyqchat/Wavs/Chatline", def_snd_chatline);
	snd_beep = cfg.readEntry("/vyqchat/Wavs/Beep", def_snd_beep);
	snd_message = cfg.readEntry("/vyqchat/Wavs/Message", def_snd_message);
	snd_join = cfg.readEntry("/vyqchat/Wavs/Join", def_snd_join);
	snd_leave = cfg.readEntry("/vyqchat/Wavs/Leave", def_snd_leave);
	snd_regexp = cfg.readEntry("/vyqchat/Wavs/RegExpMatch", def_snd_regexp);

	x = cfg.readNumEntry("/vyqchat/Window/x", -1);
	y = cfg.readNumEntry("/vyqchat/Window/y", -1);
	width = cfg.readNumEntry("/vyqchat/Window/Width", 480);
	height = cfg.readNumEntry("/vyqchat/Window/Height", 360);
}/*}}}*/

void Settings::save()/*{{{*/
{	
	if (amIroot())
		return;
	QSettings cfg;
	cfg.writeEntry("/vyqchat/General/Nick", nick);
	cfg.writeEntry("/vyqchat/General/Hostname", hostname);
	cfg.writeEntry("/vyqchat/General/IP", ip);
	cfg.writeEntry("/vyqchat/General/Broadcast", broadcast);
	cfg.writeEntry("/vyqchat/General/Refresh", refresh);
	cfg.writeEntry("/vyqchat/General/HideChannels", hide_channels);
	cfg.writeEntry("/vyqchat/General/RegExpOn", regexp_on);
	cfg.writeEntry("/vyqchat/General/Topic", topic);
	cfg.writeEntry("/vyqchat/Sound/sound", (int)sound);
	cfg.writeEntry("/vyqchat/Wavs/Chatline", snd_chatline);
	cfg.writeEntry("/vyqchat/Wavs/Join", snd_join);
	cfg.writeEntry("/vyqchat/Wavs/Leave", snd_leave);
	cfg.writeEntry("/vyqchat/Wavs/Beep", snd_beep);
	cfg.writeEntry("/vyqchat/Wavs/Message", snd_message);
	cfg.writeEntry("/vyqchat/Window/x", x);
	cfg.writeEntry("/vyqchat/Window/y", y);
	cfg.writeEntry("/vyqchat/Window/Width", width);
	cfg.writeEntry("/vyqchat/Window/Height", height);
}/*}}}*/

SettingsDialog::SettingsDialog(QWidget *parent, Settings *settings): QDialog(parent, NULL, true)/*{{{*/
{

	this->settings = settings;
	
	setCaption("VyQchat settings");
	//setFixedSize(380, 280);

	//QHBox *hbox1 = new QHBox(this);
	QBoxLayout *vbox = new QVBoxLayout(this, 4, 2);
	QBoxLayout *hbox1 = new QHBoxLayout(vbox);
	QBoxLayout *hbox2 = new QHBoxLayout(vbox);
	QBoxLayout *hbox3 = new QHBoxLayout(vbox); //for ok,defaults,cancel buttons
	
	//
	// General
	QGroupBox *opt_general = new QGroupBox(6, Qt::Vertical, "General", this);
	hbox1->addWidget(opt_general);
	//
	// Nick name validator
	QRegExp rx("[a-zA-Z]*");
	QRegExpValidator *v_nick = new QRegExpValidator(rx, this);

	QHBox *hbox1_1 = new QHBox(opt_general);
	new QLabel("Nickname", hbox1_1);
	le_nick = new QLineEdit(settings->nick, hbox1_1);
	le_nick->setMinimumWidth(140);
	le_nick->setValidator(v_nick);
	
	QHBox *hbox1_2 = new QHBox(opt_general);
	new QLabel("Hostname", hbox1_2);
	le_hostname = new QLineEdit(settings->hostname, hbox1_2);
	le_hostname->setMinimumWidth(140);

	QHBox *hbox1_3 = new QHBox(opt_general);
	new QLabel("IP", hbox1_3);
	le_ip = new QLineEdit(settings->ip, hbox1_3);
	le_ip->setMinimumWidth(100);
	QToolTip::add(le_ip, "IP address of your network interface, e.g. 192.168.0.1");
	
	QHBox *hbox1_4 = new QHBox(opt_general);
	new QLabel("Broadcast", hbox1_4);
	le_broadcast = new QLineEdit(settings->broadcast, hbox1_4);
	le_broadcast->setMinimumWidth(100);
	QToolTip::add(le_broadcast, "Broadcast address of your network interface, e.g. 192.168.0.255");
	
	QHBox *hbox1_5 = new QHBox(opt_general);
	new QLabel("Users-List Refresh", hbox1_5);
	sp_refresh = new QSpinBox(0, 300, 30, hbox1_5);
	sp_refresh->setValue(settings->refresh);
	sp_refresh->setMinimumWidth(100);
	sp_refresh->setSuffix(" sec.");
	QToolTip::add(sp_refresh, "Update interval for users list (0 for no update)");
	
	//
	// Look
	QGroupBox *opt_look = new QGroupBox(4, Qt::Vertical, "Look", this);
	hbox1->addWidget(opt_look);

	//
	// Sound device
	opt_snddev = new QVButtonGroup("Sound device", this);
	hbox2->addWidget(opt_snddev);
	QRadioButton *rb_nosnd = new QRadioButton("No sound", opt_snddev);
	QRadioButton *rb_pcs = new QRadioButton("PC speaker", opt_snddev);
	QRadioButton *rb_arts = new QRadioButton("aRtS", opt_snddev);
	QRadioButton *rb_esd = new QRadioButton("ESound", opt_snddev);
#	ifndef WITH_ARTS
	rb_arts->setDisabled(true);
#	endif
#	ifndef WITH_ESD
	rb_esd->setDisabled(true);
#	endif
	switch (settings->sound)
	{
		case Sound::NoSound: rb_nosnd->setChecked(true); break;
		case Sound::PCSpeaker: rb_pcs->setChecked(true); break;
		case Sound::Arts: rb_arts->setChecked(true); break;
		case Sound::Esd: rb_esd->setChecked(true); break;
		default: rb_pcs->setChecked(true); break;
	}
	QToolTip::add(opt_snddev, "Preffered sound output");
	
	//
	// Sounds
	QGroupBox *opt_sounds = new QGroupBox(6, Qt::Vertical, "Sounds", this);
	hbox2->addWidget(opt_sounds);
	
	QHBox *hbox2_1 = new QHBox(opt_sounds);
	new QLabel("Chatline", hbox2_1);
	QLineEdit *le_chatlinewav = new QLineEdit(settings->snd_chatline, hbox2_1);
	QPushButton *b_chatlinewav = new QPushButton("...", hbox2_1);

	QHBox *hbox2_2 = new QHBox(opt_sounds);
	new QLabel("Join channel", hbox2_2);
	QLineEdit *le_joinwav = new QLineEdit(settings->snd_join, hbox2_2);
	QPushButton *b_joinwav = new QPushButton("...", hbox2_2);
	
	QHBox *hbox2_3 = new QHBox(opt_sounds);
	new QLabel("Leave channel", hbox2_3);
	QLineEdit *le_leavenwav = new QLineEdit(settings->snd_leave, hbox2_3);
	QPushButton *b_leavewav = new QPushButton("...", hbox2_3);
	
	QHBox *hbox2_4 = new QHBox(opt_sounds);
	new QLabel("Beep", hbox2_4);
	QLineEdit *le_beepwav = new QLineEdit(settings->snd_beep, hbox2_4);
	QPushButton *b_beepewav = new QPushButton("...", hbox2_4);
	
	QHBox *hbox2_5 = new QHBox(opt_sounds);
	new QLabel("Message", hbox2_5);
	QLineEdit *le_messagewav = new QLineEdit(settings->snd_message, hbox2_5);
	QPushButton *b_messagewav = new QPushButton("...", hbox2_5);
	
	QHBox *hbox2_6 = new QHBox(opt_sounds);
	new QLabel("RegExp match", hbox2_6);
	QLineEdit *le_regexpwav = new QLineEdit(settings->snd_regexp, hbox2_6);
	QPushButton *b_regexpwav = new QPushButton("...", hbox2_6);

	//
	// Buttons
	QPushButton *b_ok = new QPushButton("OK", this);
	QPushButton *b_def = new QPushButton("Defaults", this);
	QPushButton *b_cancel = new QPushButton("Cancel", this);
	b_ok->setDefault(true);
	hbox3->addWidget(b_ok);
	hbox3->addWidget(b_def);
	hbox3->addWidget(b_cancel);
	
	connect(b_ok, SIGNAL(clicked()), this, SLOT(slotApply()));
	connect(b_cancel, SIGNAL(clicked()), this, SLOT(close()));
	connect(b_def, SIGNAL(clicked()), this, SLOT(slotDefaults()));
}/*}}}*/

SettingsDialog::~SettingsDialog()/*{{{*/
{
}
/*}}}*/

void SettingsDialog::slotApply()/*{{{*/
{
	settings->nick = le_nick->text();
	settings->hostname = le_hostname->text();
	settings->ip = le_ip->text();
	settings->broadcast = le_broadcast->text();
	settings->refresh = sp_refresh->value();
	settings->sound = (Sound::SoundType)opt_snddev->id(opt_snddev->selected());

	close();
}/*}}}*/

void SettingsDialog::slotDefaults()/*{{{*/
{
	le_nick->setText(settings->def_nick);
	le_hostname->setText(settings->def_hostname);
	le_ip->setText(settings->def_ip);
	le_broadcast->setText(settings->def_broadcast);
	sp_refresh->setValue(300);
	opt_snddev->setButton(0);
}/*}}}*/

