#include "config.h"
#include "sound.h"
#include "settings.h"
#include <qapplication.h>
#include <qfile.h>
#include <iostream.h>

#ifdef WITH_ARTS
#include <artsc.h>
#endif

QString Sound::errstr;
Sound::SoundType Sound::sndtype;
QByteArray Sound::sample[6];

bool Sound::load_sample(Sfx sfx, const QString &filename)/*{{{*/
{
	QFile f(filename); 
	if (f.open(IO_ReadOnly))
	{ 
		sample[sfx] = QByteArray(f.size());
		f.readBlock(sample[sfx].data(), f.size());
		f.close();
	}
	return true;
}/*}}}*/

bool Sound::init(Settings *settings)/*{{{*/
{
	//
	// load wav files
	load_sample(Chatline, settings->snd_chatline);
	load_sample(JoinChannel, settings->snd_join);
	load_sample(LeaveChannel, settings->snd_leave);
	load_sample(Beep, settings->snd_beep);
	load_sample(Message, settings->snd_message);
		
	sndtype = settings->sound;
	switch (sndtype)
	{
		case NoSound:
		case PCSpeaker:
			break;
		case Arts:
#		ifdef WITH_ARTS
			int err;
			if ((err = arts_init()) < 0)
			{
				errstr = QString(arts_error_text(err));
				sndtype = PCSpeaker;
				return false;
			}
			break;
#		else
			errstr = "no aRtS support compiled";
			return false;
#		endif
		case Esd: 
			break;
#		ifdef WITH_ESD
			;
#		else
			errstr = "no ESound support compiled";
			sndtype = PCSpeaker;
			return false;
#		endif
		default: return true;
	}
	return true;
}/*}}}*/

void Sound::close()/*{{{*/
{
	switch (sndtype)
	{
#	ifdef WITH_ARTS
		case Arts: arts_free(); break;
#	endif
#	ifdef WITH_ESD
		case Esd: break;
#	endif
		default: break;
	}
}/*}}}*/

void Sound::play(Sfx sfx)/*{{{*/
{
	switch (sndtype)
	{
		case NoSound: break;
		case PCSpeaker: QApplication::beep(); break;
#		ifdef WITH_ARTS
		case Arts: { 
				   arts_stream_t stream = arts_play_stream(22050, 8, 1, "vyqchat");
				   int status = arts_write(stream, sample[sfx].data(), sample[sfx].size());
				   if(status < 0)
					   cout << "arts_write error: " <<  arts_error_text(status) << endl;
				   arts_close_stream(stream);
			   }
				break;
#		endif
#		ifdef WITH_ESD
		case Esd: break;
#		endif
	}
}/*}}}*/

