#include "ulist.h"
#include "vypacket.h"
#include "msgwin.h"
#include "icons.h"
#include <qtooltip.h>

User::User(QListBox *list, const QString &nick, const QHostAddress &addr): QListBoxItem(list)/*{{{*/
{
	ip = addr;
	dcnt = 1;
	setNick(nick);
	setStatus(STATUS_NORMAL);
}/*}}}*/

User::~User()/*{{{*/
{
}/*}}}*/

void User::paint(QPainter *p)/*{{{*/
{
	switch (status)
	{
		case STATUS_NORMAL : p->drawPixmap(0, 0, Icons::icon_user.pixmap(QIconSet::Small, true)); break;
		case STATUS_DND    : p->drawPixmap(0, 0, Icons::icon_user_dnd.pixmap(QIconSet::Small, true)); break;
		case STATUS_AWAY   : p->drawPixmap(0, 0, Icons::icon_user_away.pixmap(QIconSet::Small, true)); break;
		case STATUS_OFFLINE: p->drawPixmap(0, 0, Icons::icon_user_offline.pixmap(QIconSet::Small, true)); break;
		default: break;
	}
	p->drawText(Icons::icon_user.pixmap().width() + 2, p->fontMetrics().ascent(), text());
}/*}}}*/

int User::height(const QListBox *lb) const/*{{{*/
{
	return QMAX(lb->fontMetrics().height(), Icons::icon_user.pixmap(QIconSet::Small, true).height());
}/*}}}*/

int User::width(const QListBox *lb) const/*{{{*/
{
	return Icons::icon_user.pixmap(QIconSet::Small, true).width() + lb->fontMetrics().width(text()) + 2;
}
	
const QPixmap* User::pixmap() const
{
	return NULL;
}/*}}}*/

void User::setToolTipInfo(const QString &info)
{
	//QToolTip::add(this, info);
}

UsersList::UsersList(QWidget *parent): QListBox(parent)/*{{{*/
{
	setMinimumWidth(120);
	setFocusPolicy(QWidget::NoFocus);
	setSelectionMode(QListBox::Single);
}/*}}}*/

UsersList::~UsersList()/*{{{*/
{
}/*}}}*/

bool UsersList::newNick(const QString &onick, const QString &nnick)/*{{{*/
{
	User *user = getUser(onick);
	if (user)
	{
		user->setNick(nnick);
		umap.remove(onick);
		umap[nnick] = user;
		updateItem(user);
		return true;
	}
	return false;
}/*}}}*/

User *UsersList::addUser(const QString &nick, char gender, char status, const QHostAddress &addr)/*{{{*/
{
	if (exists(nick))
		return NULL;
	
	User *u = new User(this, nick, addr);
	u->setGender(gender);
	u->setStatus(status);
	QListBox::sort();
	return umap[nick] = u;
}/*}}}*/

bool UsersList::delUser(const QString &nick)/*{{{*/
{
	User *user = getUser(nick);
	if (user)
	{
		umap.remove(nick);
		delete user;
		return true;
	}
	return false;
}/*}}}*/

char UsersList::getStatus(const QString &nick)/*{{{*/
{
	User *user = getUser(nick);
	return user!=NULL ? user->getStatus() : STATUS_UNKNOWN; 
}/*}}}*/

bool UsersList::setStatus(const QString &nick, char s)/*{{{*/
{
	User *user = getUser(nick);
	return user!=NULL ? updateItem(user), setStatus(user, s) : false;
}/*}}}*/

void UsersList::decDeadCounters()/*{{{*/
{
	QStringList l;
	//
	// create a temporary list of non-existent users;
	for (UserMap::iterator it = umap.begin(); it!=umap.end(); ++it)
		if (it.data()->decDeadCounter() < 0)
			l.append(it.key());
	//
	// delete all users from list
	if (!l.empty())
		for (QStringList::iterator it=l.begin(); it!=l.end(); ++it)
			delUser(*it);
}/*}}}*/

