#ifndef __ULIST_H
#define __ULIST_H

#include <qhostaddress.h>
#include <qlistbox.h>
#include <qmap.h>
#include <qpopupmenu.h>
#include <qpainter.h>

class User: public QListBoxItem
{
	private:
		char gender;
		char status;
		QHostAddress ip;
		int dcnt;
		QString nick;
		
		bool setGender(char g) { gender=g; return true; }
		bool User::setStatus(char s) { status=s; return true; }
		void setNick(const QString &nick) { setText(nick); }
		int incDeadCounter() { if (dcnt<1) dcnt++; return dcnt; }
		int decDeadCounter() { return --dcnt; }
	public:
		User(QListBox *list, const QString &nick, const QHostAddress &addr);
		~User();
		char getGender() { return gender; }
		char getStatus() { return status; }
		QString getNick() { return text(); }
		QHostAddress getIP() { return ip; }
		void paint(QPainter * p);
		int height(const QListBox * lb) const;
		int width(const QListBox * lb) const;
		const QPixmap* pixmap() const;
		void setToolTipInfo(const QString &info);
		friend class UsersList;
};

class UsersList: public QListBox
{
	private:
		typedef QMap<QString, User *> UserMap;
		UserMap umap;
	public:
		UsersList(QWidget *);
		~UsersList();
		bool exists(const QString &nick) { return umap.contains(nick); }
		User *getUser(const QString &nick) { return exists(nick) ? umap[nick] : NULL; }
		bool newNick(const QString &onick, const QString &nnick);
		User *addUser(const QString &nick, char gender, char status, const QHostAddress &addr);
		bool delUser(const QString &nick);
		QHostAddress getIP(const QString &nick) { return umap.contains(nick) ? umap[nick]->getIP() : QHostAddress(); }
		char getStatus(const QString &nick);
		char getStatus(User *user) { return user->getStatus(); }
		bool setStatus(const QString &nick, char s);
		bool setStatus(User *user, char s) { return user->setStatus(s); }
		void clear() { umap.clear(); QListBox::clear(); }
		void decDeadCounters();
		int incDeadCounter(const QString &nick) { User *u=getUser(nick); return u!=NULL ? u->incDeadCounter() : -1; }
};

#endif

