#include "vypacket.h"
#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <assert.h>

VyPacket::VyPacket(char *p_data, int p_len):/*{{{*/
	channel(NULL), from(NULL), to(NULL), status(0), gender(0), text(NULL)
{
	assert(p_data);
	buffer = new char [p_len];
	memcpy(buffer, p_data, p_len);
	decode(buffer, p_len);
}/*}}}*/

VyPacket::~VyPacket()/*{{{*/
{
	delete [] buffer;
}/*}}}*/

char** VyPacket::split(char *ptr[], char *buff, int num)/*{{{*/
{
	for (int i=0; i<num; i++)
	{
		ptr[i] = buff; //store pointer to the beginning of substring
		for (; *buff; buff++);    //move to the end of this substring
		buff++;                //skip zero byte
	}
	return ptr;
}/*}}}*/

bool VyPacket::decode(char *p_data, int p_len)/*{{{*/
{
	char *ptrs[8];
	char *p = p_data;
	p += 10; 		//skip packet id
	char b = *(p++);
	switch (b)
	{
		/* WHO HERE
	 	* updater */
		case VY_WHOHERE:
			from = p;
			ptype = VyWhoHere;
			break;
		/* IM HERE
		 * updater :: responder :: status + remote active */
		case VY_IMHERE:
			split(ptrs, p, 3);
			to = ptrs[0];
			from = ptrs[1];
			status = ptrs[2][0];
			remoteactive = ptrs[2][1];
			ptype = VyImHere;
			break;
		/* SIMPLE CHAT
		 * channel :: author :: text */
		case VY_SAY: 
			split(ptrs, p, 3);
			from = ptrs[1];
			channel = ptrs[0];
			text = ptrs[2];
			ptype = VySay;
			break;
		/* ME
		 * channel :: author :: text */
		case VY_ME:
			split(ptrs, p, 3);
			channel = ptrs[0];
			from = ptrs[1];
			text = ptrs[2];
			ptype = VyMe;
			break;
		/* NICK CHANGE
		 * old :: new :: gender */
		case VY_NICKCHANGE: 
			split(ptrs, p, 2);
			from = ptrs[0];
			to = ptrs[1];
			gender = ptrs[2][0];
			ptype = VyNickChange;
			break;
		case VY_BEEP:
			  b = *(p++);
			  switch (b)
			  {
				  /* BEEP SIGNAL
				   * to :: from  */
				  case VY_BEEP_SIGNAL: 
					  split(ptrs, p, 2);
					  to = ptrs[0];
					  from = ptrs[1];
					  ptype = VyBeepSignal;
					  break;
				  /* BEEP ACK
				   * to :: from :: gender */
				  case VY_BEEP_ACK: 
					  split(ptrs, p, 3);
					  to = ptrs[0];
					  from = ptrs[1];
					  gender = ptrs[2][0];
					  ptype = VyBeepAck;
					  break;
				  default: break;
			  }
			  break;
		/* JOIN
		 * who :: channel :: status + gender */
		case VY_JOIN: 
			  split(ptrs, p, 3);
			  from = ptrs[0];
			  channel = ptrs[1];
			  status = ptrs[2][0];
			  gender = ptrs[2][1];
			  ptype = VyJoin;
			  break;
		/* LEAVE
		 * who :: channel :: gender */
		case VY_LEAVE:
			  split(ptrs, p, 3);
			  from = ptrs[0];
			  channel = ptrs[1];
			  gender = ptrs[2][0];
			  ptype = VyLeave;
			  break;	  
		/* PRIVATE */
		case VY_PRIVATE:
			  b = *(p++);
			  split(ptrs, p, 3);
			  from = ptrs[0];
			  to = ptrs[1];
			  switch (b)
			  {
				  /* PRIVATE JOIN, PRIVATE_LEAVE
				   * from :: to :: gender */
				  case VY_PRIVATE_JOIN:	
					  	gender = ptrs[2][0];
					  	ptype = VyPrivateJoin;
						break;
				  case VY_PRIVATE_LEAVE:
					  	gender = ptrs[2][0];
						ptype = VyPrivateLeave;
						break;
				  /* PRIVATE_SAY, PRIVATE_ME
				   * from :: to :: text */
				  case VY_PRIVATE_SAY:
						text = ptrs[2];
						ptype = VyPrivateSay;
						break;
				  case VY_PRIVATE_ME:  
						text = ptrs[2];
						ptype = VyPrivateMe;
						break;
				  default: break;
			  }
			  break;
		/* MESSAGE
		 * from :: to :: text */
		case VY_MESSAGE: 
			  split(ptrs, p, 3);
			  from = ptrs[0];
			  to = ptrs[1];
			  text = ptrs[2];
			  ptype = VyMessage;
			  break;
		/* MESSAGEACK
		 * status + whorec :: towhoack :: gender + currentaa */	  
		case VY_MESSAGEACK:
			  split(ptrs, p, 3);
			  status = ptrs[0][0];
			  from = ptrs[0]+1;
			  to = ptrs[1];
			  gender = ptrs[2][0];
			  remoteactive = ptrs[2][1];
			  ptype = VyMessageAck;
			  break;
		/* HERE REQUEST
		 * from :: channel */
		case VY_HEREREQ:
			  split(ptrs, p, 2);
			  from = ptrs[0];
			  channel = ptrs[1];
			  ptype = VyHereReq;
			  break;
		/* HERE ACK 
		 * to :: channel :: from :: remote active */
		case VY_HEREACK:
			  split(ptrs, p, 4);
			  to = ptrs[0];
			  channel = ptrs[1];
			  from = ptrs[2];
			  remoteactive = ptrs[3][0];
			  ptype = VyHereReqAck;
			  break;
		/* INFO REQUEST
		 * to :: from */
		case VY_INFOREQ: 
			  split(ptrs, p, 2);
			  to = ptrs[0];
			  from = ptrs[1];
			  ptype = VyInfoReq;
			  break;
		/* INFO REQUEST ACK
		 * to :: from :: computer :: name :: ip :: channels :: answer */
		case VY_INFOREQACK:  
			  split(ptrs, p, 7);
			  to = ptrs[0];
			  from = ptrs[1];
			  computer = ptrs[2];
			  name = ptrs[3];
			  ip = ptrs[4];
			  channel = ptrs[5];
			  text = ptrs[6];
			  ptype = VyInfoReqAck;
			  break;
	 	/* CURRENT TOPIC
		 * towho :: channel :: topic */
		case VY_CURRTOPIC: 
			  split(ptrs, p, 3);
			  to = ptrs[0];
			  channel = ptrs[1];
			  topic = ptrs[2];
			  ptype = VyCurrTopic;
			  break;
		/* NEW TOPIC
		 * channel :: topic */
		case VY_NEWTOPIC:
			  split(ptrs, p, 2);
			  channel = ptrs[0];
			  topic = ptrs[1];
			  ptype = VyNewTopic;
			  break;
		/* CHANGE STATUS
		 * who :: status + gender + aa */
		case VY_CHANGESTAT:
			  split(ptrs, p, 2);
			  from = ptrs[0];
			  status = ptrs[1][0];
			  gender = ptrs[1][1];
			  text = ptrs[1]+2;
			  ptype = VyChangeStat;
			  break;
		default:
			cerr << "packet " << b << " not implemented (yet)" << endl;

			break;
	}
	return true;
}/*}}}*/

